/*************************************************************************
  File Name          : orion_firmware.ino
  Author             : Ander, Mark Yan
  Updated            : Ander, Mark Yan
  Version            : V0a.01.105
  Date               : 07/06/2016
  Description        : Firmware for Makeblock Electronic modules with Scratch.
  License            : CC-BY-SA 3.0
  Copyright (C) 2013 - 2016 Maker Works Technology Co., Ltd. All right reserved.
  http://www.makeblock.cc/

  Version 0.4.18.2.1 
**************************************************************************/
#include <Wire.h>
#include <SoftwareSerial.h>
#include <Arduino.h>
#include <GeekeBrain.h> 
#include <GkBuzzer.h>

Servo servos[8];

/*
MeDCMotor dc;
MeTemperature ts;
MeRGBLed led;
MeUltrasonicSensor us;
Me7SegmentDisplay seg;
MePort generalDevice;
MeInfraredReceiver *ir = NULL;
MeGyro gyro;
MeJoystick joystick;
MeStepper steppers[2];
MeHumiture humiture;
MeFlameSensor FlameSensor;
MeGasSensor GasSensor;
MeTouchSensor touchSensor;
Me4Button buttonSensor;
*/
GkBuzzer buzzer;

typedef struct MeModule{
  int device;
  int port;
  int slot;
  int pin;
  int index;
  float values[3];
} MeModule;

union {
  byte byteVal[4];
  float floatVal;
  long longVal;
} val;

union {
  byte byteVal[8];
  double doubleVal;
} valDouble;

union {
  byte byteVal[2];
  short shortVal;
} valShort;

MeModule modules[12];

#if defined(__AVR_ATmega32U4__)
int analogs[12] = {A0, A1, A2, A3, A4, A5, A6, A7, A8, A9, A10, A11};
#endif
#if defined(__AVR_ATmega328P__) or defined(__AVR_ATmega168__)
int analogs[8] = {A0, A1, A2, A3, A4, A5, A6, A7};
//ººMeEncoderMotor encoders[2];
#endif
#if defined(__AVR_ATmega1280__)|| defined(__AVR_ATmega2560__)
int analogs[16] = {A0, A1, A2, A3, A4, A5, A6, A7, A8, A9, A10, A11, A12, A13, A14, A15};
#endif
String mVersion = "0a.01.105";
boolean isAvailable = false;
boolean isBluetooth = false;

int len = 52;
char buffer[52];
char bufferBt[52];
byte index = 0;
byte dataLen;
byte modulesLen = 0;
boolean isStart = false;
unsigned char irRead;
char serialRead;
#define VERSION 0
#define ULTRASONIC_SENSOR 1
#define TEMPERATURE_SENSOR 2
#define LIGHT_SENSOR 3
#define POTENTIONMETER 4
#define JOYSTICK 5
#define GYRO 6
#define SOUND_SENSOR 7
#define RGBLED 8
#define SEVSEG 9
#define MOTOR 10
#define SERVO 11
#define ENCODER 12
#define IR 13
#define IRREMOTE 14
#define PIRMOTION 15
#define INFRARED 16
#define LINEFOLLOWER 17
#define IRREMOTECODE 18
#define SHUTTER 20
#define LIMITSWITCH 21
#define BUTTON 22
#define HUMITURE 23
#define FLAMESENSOR 24
#define GASSENSOR 25
#define COMPASS 26
#define DIGITAL 30
#define ANALOG 31
#define PWM 32
#define SERVO_PIN 33
#define TONE 34
#define PULSEIN 37
#define ULTRASONIC_ARDUINO 36
#define STEPPER 40
#define LEDMATRIX 41
#define TIMER 50
#define TOUCH_SENSOR 51

//#define GK_MOTOR 100

#define GET 1
#define RUN 2
#define RESET 4
#define START 5
float angleServo = 90.0;
int servo_pins[8] = {0, 0, 0, 0, 0, 0, 0, 0};
unsigned char prevc = 0;
double lastTime = 0.0;
double currentTime = 0.0;
uint8_t keyPressed = 0;
uint8_t command_index = 0;

void setup() {
  pinMode(13, OUTPUT);
  blinkLed(2);
  
  Serial.begin(115200);
  delay(500);
  buzzer.tone(8, 250, 300);
  buzzer.tone(8, 300, 300);

#if defined(__AVR_ATmega328P__) or defined(__AVR_ATmega168__)
/*ºº  encoders[0] = MeEncoderMotor(SLOT_1);
  encoders[1] = MeEncoderMotor(SLOT_2);
  encoders[0].begin();
  encoders[1].begin();
  delay(500);
  encoders[0].runSpeed(0);
  encoders[1].runSpeed(0);
ºº*/  
#else
  Serial1.begin(115200);
#endif
  //gyro.begin();
  Serial.print("Version: ");
  Serial.println(mVersion);

  blinkLed(2);  
}

void loop() {
//  keyPressed = buttonSensor.pressed();
  currentTime = millis() / 1000.0 - lastTime;
  /*ºº
  if (ir != NULL)
  {
    ir->loop();
  }
  */
  readSerial();
//ºº  steppers[0].runSpeedToPosition();
//ºº  steppers[1].runSpeedToPosition();
  if (isAvailable) {
    unsigned char c = serialRead & 0xff;
    if (c == 0x55 && isStart == false) {
      if (prevc == 0xff) {
        index = 1;
        isStart = true;
      }
    } else {
      prevc = c;
      if (isStart) {
        if (index == 2) {
          dataLen = c;
        } else if (index > 2) {
          dataLen--;
        }
        writeBuffer(index, c);
      }
    }
    index++;
    if (index > 51) {
      index = 0;
      isStart = false;
    }
    if (isStart && dataLen == 0 && index > 3) {
      isStart = false;
      parseData();
      index = 0;
    }
  }

}

unsigned char readBuffer(int index) {
  return isBluetooth ? bufferBt[index] : buffer[index];
}

void writeBuffer(int index, unsigned char c) {
  if (isBluetooth) {
    bufferBt[index] = c;
  } else {
    buffer[index] = c;
  }
}

void writeHead() {
  writeSerial(0xff);
  writeSerial(0x55);
}

void writeEnd() {
  Serial.println();
#if defined(__AVR_ATmega32U4__)
  Serial1.println();
#endif
}

void writeSerial(unsigned char c) {
  Serial.write(c);
#if defined(__AVR_ATmega32U4__)
  Serial1.write(c);
#endif
}

void readSerial() {
  isAvailable = false;
  if (Serial.available() > 0) {
    isAvailable = true;
    isBluetooth = false;
    serialRead = Serial.read();
  }
  //#if defined(__AVR_ATmega32U4__)
  //  if(Serial1.available()>0){
  //    isAvailable = true;
  //    isBluetooth = false;
  //    serialRead = Serial1.read();
  //  }
  // #endif
}

/*
  ff 55 len idx action device port  slot  data a
  0  1  2   3   4      5      6     7     8
*/
void parseData() {
  //blinkLed(2);
  
  isStart = false;
  int idx = readBuffer(3);
  command_index = (uint8_t)idx;
  int action = readBuffer(4);
  int device = readBuffer(5);
  switch (action) {
    case GET: {
        if (device != ULTRASONIC_SENSOR) {
          writeHead();
          writeSerial(idx);
        }
        readSensor(device);
        writeEnd();
      }
      break;

      
    case RUN: {
        runModule(device);
        callOK();
      }
      break;

      
    case RESET: {
        /*
        //reset
        dc.reset(M1);
        dc.run(0);
        dc.reset(M2);
        dc.run(0);
        dc.reset(PORT_1);
        dc.run(0);
        dc.reset(PORT_2);
        dc.run(0);
        */
                
#if defined(__AVR_ATmega328P__)
//ºº        encoders[0].runSpeed(0);
//ºº        encoders[1].runSpeed(0);
#endif
        callOK();
      }
      break;
    case START: {
        //start
        callOK();
      }
      break;
  }
}


void callOK() {
  writeSerial(0xff);
  writeSerial(0x55);
  writeEnd();
}
void sendByte(char c) {
  writeSerial(1);
  writeSerial(c);
}
void sendString(String s) {
  int l = s.length();
  writeSerial(4);
  writeSerial(l);
  for (int i = 0; i < l; i++) {
    writeSerial(s.charAt(i));
  }
}
void sendFloat(float value) {
  writeSerial(0x2);
  val.floatVal = value;
  writeSerial(val.byteVal[0]);
  writeSerial(val.byteVal[1]);
  writeSerial(val.byteVal[2]);
  writeSerial(val.byteVal[3]);
}
void sendShort(double value) {
  writeSerial(3);
  valShort.shortVal = value;
  writeSerial(valShort.byteVal[0]);
  writeSerial(valShort.byteVal[1]);
}
void sendDouble(double value) {
  writeSerial(2);
  valDouble.doubleVal = value;
  writeSerial(valDouble.byteVal[0]);
  writeSerial(valDouble.byteVal[1]);
  writeSerial(valDouble.byteVal[2]);
  writeSerial(valDouble.byteVal[3]);
}
short readShort(int idx) {
  valShort.byteVal[0] = readBuffer(idx);
  valShort.byteVal[1] = readBuffer(idx + 1);
  return valShort.shortVal;
}
float readFloat(int idx) {
  val.byteVal[0] = readBuffer(idx);
  val.byteVal[1] = readBuffer(idx + 1);
  val.byteVal[2] = readBuffer(idx + 2);
  val.byteVal[3] = readBuffer(idx + 3);
  return val.floatVal;
}
long readLong(int idx) {
  val.byteVal[0] = readBuffer(idx);
  val.byteVal[1] = readBuffer(idx + 1);
  val.byteVal[2] = readBuffer(idx + 2);
  val.byteVal[3] = readBuffer(idx + 3);
  return val.longVal;
}




// ---------------------------------------------------------------------------------
void runModule(int device) {
  /**************************************************
      ff 55 len idx action device port slot data a
      0  1  2   3   4      5      6    7    8
  ***************************************************/
  //0xff 0x55 0x6 0x0 0x1 0xa 0x9 0x0 0x0 0xa
  
  int port = readBuffer(6);
  int pin = port;

  //blinkLed(3);
  
  switch (device) {
    case MOTOR: {
        int motorDir = readShort(7);
        int motorSpeed = readShort(9);
        int motorSteeps = readShort(11);
        runMotor(motorDir, motorSpeed, motorSteeps);
    }break;

/*    
    case MOTOR: {
        int speed = readShort(7);
        dc.reset(port);
        dc.run(speed);
      }
      break;
*/

/*      
    case JOYSTICK: {
        int leftSpeed = readShort(6);
        dc.reset(M1);
        dc.run(leftSpeed);
        int rightSpeed = readShort(8);
        dc.reset(M2);
        dc.run(rightSpeed);
      }
      break;
*/

/*      
    case STEPPER: {
        int maxSpeed = readShort(7);
        long distance = readLong(9);
        if (port == PORT_1) {
          steppers[0] = MeStepper(PORT_1);
          steppers[0].moveTo(distance);
          steppers[0].setMaxSpeed(maxSpeed);
          steppers[0].setSpeed(maxSpeed);
        } else if (port == PORT_2) {
          steppers[1] = MeStepper(PORT_2);
          steppers[1].moveTo(distance);
          steppers[1].setMaxSpeed(maxSpeed);
          steppers[1].setSpeed(maxSpeed);
        }
      }
      break;
*/

/*
    case ENCODER: {
        int slot = readBuffer(7);
        int maxSpeed = readShort(8);
        float distance = readFloat(10);
#if defined(__AVR_ATmega328P__)
        if (slot == SLOT_1) {
          //ººencoders[0].move(distance, maxSpeed);
        } else if (slot == SLOT_2) {
          //ººencoders[1].move(distance, maxSpeed);
        }
#endif
      }
      break;
*/

/*      
    case RGBLED: {
        int slot = readBuffer(7);
        int idx = readBuffer(8);
        int r = readBuffer(9);
        int g = readBuffer(10);
        int b = readBuffer(11);
        led.reset(port, slot);
        if (idx > 0)
        {
          led.setColorAt(idx - 1, r, g, b);
        }
        else
        {
          led.setColor(r, g, b);
        }
        led.show();
      }
      break;
*/

/*      
    case SERVO: {
        int slot = readBuffer(7);
        pin = slot == 1 ? mePort[port].s1 : mePort[port].s2;
        int v = readBuffer(8);
        Servo sv = servos[searchServoPin(pin)];
        if (v >= 0 && v <= 180)
        {
          if (!sv.attached())
          {
            sv.attach(pin);
          }
          sv.write(v);
        }
      }
      break;
*/

/*      
    case SEVSEG: {
        if (seg.getPort() != port) {
          seg.reset(port);
        }
        float v = readFloat(7);
        seg.display(v);
      }
      break;
*/

/*      
    case LIGHT_SENSOR: {
        if (generalDevice.getPort() != port) {
          generalDevice.reset(port);
        }
        int v = readBuffer(7);
        generalDevice.dWrite1(v);
      }
      break;
*/

/*      
    case SHUTTER: {
        if (generalDevice.getPort() != port) {
          generalDevice.reset(port);
        }
        int v = readBuffer(7);
        if (v < 2) {
          generalDevice.dWrite1(v);
        } else {
          generalDevice.dWrite2(v - 2);
        }
      }
      break;
*/
      
    case DIGITAL: {
        pinMode(pin, OUTPUT);
        int v = readBuffer(7);
        digitalWrite(pin, v);
      }
      break;

      
    case PWM: {
        pinMode(pin, OUTPUT);
        int v = readBuffer(7);
        analogWrite(pin, v);
      }
      break;


      
    case TONE: {
        pinMode(pin, OUTPUT);
        int hz = readShort(7);
        int ms = readShort(9);
        if (ms > 0) {
          buzzer.tone(pin, hz, ms);
        } else {
          buzzer.noTone(pin);
        }
      }
      break;

      
    case SERVO_PIN: {
        int v = readBuffer(7);
        Servo sv = servos[searchServoPin(pin)];
        if (v >= 0 && v <= 180)
        {
          if (!sv.attached())
          {
            sv.attach(pin);
          }
          sv.write(v);
        }
      }
      break;
      
    case TIMER: {
        lastTime = millis() / 1000.0;
      }
      break;
  }
}

int searchServoPin(int pin) {
  for (int i = 0; i < 8; i++) {
    if (servo_pins[i] == pin) {
      return i;
    }
    if (servo_pins[i] == 0) {
      servo_pins[i] = pin;
      return i;
    }
  }
  return 0;
}

void readSensor(int device) {
  /**************************************************
      ff 55 len idx action device port slot data a
      0  1  2   3   4      5      6    7    8
  ***************************************************/
  float value = 0.0;
  int port, slot, pin;
  port = readBuffer(6);
  pin = port;
  switch (device) {

    /* Sensor de MakeBlock
    case  ULTRASONIC_SENSOR: {
        GkUltrasonicSensor us;
        blinkLed(3);
        if (us.getPort() != port) {
          us.reset(port);
        }
        value = us.distanceCm();
        writeHead();
        writeSerial(command_index);
        sendShort(value);
      }
      break;
    */


/*      
    case  TEMPERATURE_SENSOR: {
        slot = readBuffer(7);
        if (ts.getPort() != port || ts.getSlot() != slot) {
          ts.reset(port, slot);
        }
        value = ts.temperature();
        sendFloat(value);
      }
      break;
*/

/*      
    case  LIGHT_SENSOR:
    case  SOUND_SENSOR:
    case  POTENTIONMETER: {
        if (generalDevice.getPort() != port) {
          generalDevice.reset(port);
          pinMode(generalDevice.pin2(), INPUT);
        }
        value = generalDevice.aRead2();
        sendFloat(value);
      }
      break;
*/

/*      
    case  JOYSTICK: {
        slot = readBuffer(7);
        if (joystick.getPort() != port) {
          joystick.reset(port);
        }
        value = joystick.read(slot);
        sendFloat(value);
      }
      break;
*/


/*
    case  INFRARED:
      {
        if (ir == NULL)
        {
          ir = new MeInfraredReceiver(port);
          ir->begin();
        }
        else if (ir->getPort() != port)
        {
          delete ir;
          ir = new MeInfraredReceiver(port);
          ir->begin();
        }
        irRead = ir->getCode();
        if ((irRead < 255) && (irRead > 0))
        {
          sendFloat((float)irRead);
        }
        else
        {
          sendFloat(0);
        }
      }
      break;
*/

/*
    case  PIRMOTION: {
        if (generalDevice.getPort() != port) {
          generalDevice.reset(port);
          pinMode(generalDevice.pin2(), INPUT);
        }
        value = generalDevice.dRead2();
        sendFloat(value);
      }
      break;
*/

/*      
    case  LINEFOLLOWER: {
        if (generalDevice.getPort() != port) {
          generalDevice.reset(port);
          pinMode(generalDevice.pin1(), INPUT);
          pinMode(generalDevice.pin2(), INPUT);
        }
        value = generalDevice.dRead1() * 2 + generalDevice.dRead2();
        sendFloat(value);
      }
      break;
*/

/*      
    case LIMITSWITCH: {
        slot = readBuffer(7);
        if (generalDevice.getPort() != port || generalDevice.getSlot() != slot) {
          generalDevice.reset(port, slot);
        }
        if (slot == 1) {
          pinMode(generalDevice.pin1(), INPUT_PULLUP);
          value = !generalDevice.dRead1();
        } else {
          pinMode(generalDevice.pin2(), INPUT_PULLUP);
          value = !generalDevice.dRead2();
        }
        sendFloat(value);
      }
      break;
*/      
    //   case COMPASS:{
    //     if(Compass.getPort()!=port){
    //       Compass.reset(port);
    //       Compass.setpin(Compass.pin1(),Compass.pin2());
    //     }
    //     double CompassAngle;
    //     CompassAngle = Compass.getAngle();
    //     sendDouble(CompassAngle);
    //   }
    //   break;


/*    
    case HUMITURE: {
        uint8_t index = readBuffer(7);
        if (humiture.getPort() != port) {
          humiture.reset(port);
        }
        uint8_t HumitureData;
        humiture.update();
        HumitureData = humiture.getValue(index);
        sendByte(HumitureData);
      }
      break;
*/
      

/*  
    case FLAMESENSOR: {
        if (FlameSensor.getPort() != port) {
          FlameSensor.reset(port);
          FlameSensor.setpin(FlameSensor.pin2(), FlameSensor.pin1());
        }
        int16_t FlameData;
        FlameData = FlameSensor.readAnalog();
        sendShort(FlameData);
      }
      break;
*/


/*      
    case GASSENSOR: {
        if (GasSensor.getPort() != port) {
          GasSensor.reset(port);
          GasSensor.setpin(GasSensor.pin2(), GasSensor.pin1());
        }
        int16_t GasData;
        GasData = GasSensor.readAnalog();
        sendShort(GasData);
      }
      break;
*/


    /*
      case  GYRO:{
        int axis = readBuffer(7);
        gyro.update();
        if(axis == 1){
          value = gyro.getAngleX();
          sendFloat(value);
        }else if(axis == 2){
          value = gyro.getAngleY();
          sendFloat(value);
        }else if(axis == 3){
          value = gyro.getAngleZ();
          sendFloat(value);
        }
      }
      break;
    */

    case  VERSION: {
        sendString(mVersion);
      }
      break;

      
    case  DIGITAL: {
        pinMode(pin, INPUT);
        sendFloat(digitalRead(pin));
      }
      break;

      
    case  ANALOG: {
        pin = analogs[pin];
        pinMode(pin, INPUT);
        sendFloat(analogRead(pin));
      }
      break;

      
    case  PULSEIN: {
        int pw = readShort(7);
        pinMode(pin, INPUT);
        sendShort(pulseIn(pin, HIGH, pw));
      }
      break;

      
    case ULTRASONIC_ARDUINO: {
        int trig = readBuffer(6);
        int echo = trig; //readBuffer(7);
        pinMode(trig, OUTPUT);

        int distance1 = 0;
        int distance2 = 100;
        int i = 0;

        // Si da un valor 0 o da una diferencia 
        while(
          (distance1 == 0) 
          && 
          (i < 20)
        ){
            digitalWrite(trig, LOW);
            delayMicroseconds(4);
            digitalWrite(trig, HIGH);
            delayMicroseconds(10);
            digitalWrite(trig, LOW);
            
            pinMode(echo, INPUT);
            distance1 = pulseIn(echo, HIGH, 30000) / 58.0;

            i++;
            if (distance1 == 0){
                delay(10);
            }
        }
        
        sendShort(distance1);
        
    } break;

      
    case TIMER: {
        sendFloat((float)currentTime);
      }
      break;

/*      
    case TOUCH_SENSOR:
      {
        if (touchSensor.getPort() != port) {
          touchSensor.reset(port);
        }
        sendByte(touchSensor.touched());
      }
      break;
*/

      /*
        case BUTTON:
        {
          if(buttonSensor.getPort() != port){
            buttonSensor.reset(port);
          }
          sendByte(keyPressed == readBuffer(7));
        }
        break;
      */

  }
  

}


/**********************************************************************************
 * Gestion de Motor 
 */
volatile int motorRoundsWhell1 = 0;
volatile int motorRoundsWhell2 = 0;
volatile int motorRoundsWhellTarget = 0;
volatile int motor5 = 0;
volatile int motor6 = 0;
volatile int motor9 = 0;
volatile int motor10 = 0;
volatile int motorNow5 = 0;
volatile int motorNow6 = 0;
volatile int motorNow9 = 0;
volatile int motorNow10 = 0;
volatile int motorMedia5_6 = 0;
volatile int motorMedia9_10 = 0;

/*
 * Si motorTime = -1 utiliza motorSteeps
 * Si motorSteeps = -1 utiliza motorTime
 */
void runMotor(int motorDir, int motorSpeed, int motorSteeps){
    //blinkLed(motorDir);  

    motorRoundsWhell1 = 0;
    motorRoundsWhell2 = 0;
    motorRoundsWhellTarget = motorSteeps;
    // Motor Izquierda
    pinMode(5, OUTPUT);
    pinMode(6, OUTPUT);
    // Motor Derecha
    pinMode(9, OUTPUT);
    pinMode(10, OUTPUT);

    /*
    if (motorDir != 5 && motorDir != 6){
        digitalWrite(5, LOW);
        digitalWrite(6, LOW);
        digitalWrite(9, LOW);
        digitalWrite(10, LOW);
    }
    */
            
    if (motorSpeed > 0){
        // Avanzar
        if(motorDir == 1){
            motor6 = 0;
            motor9 = 0;
            motor5 = motorSpeed;
            motor10 = motorSpeed;
               
        // Derecha
        }else if(motorDir == 2 || motorDir == 6){
            motor6 = 0;
            motor5 = motorSpeed;
    
        // Izquierda
        }else if(motorDir == 3 || motorDir == 5){
            motor9 = 0;
            motor10 = motorSpeed;
    
        // Atras
        }else if(motorDir == 4){
            motor10 = 0;
            motor5 = 0;
            motor6 = motorSpeed;
            motor9 = motorSpeed;
    
        // Para
        }else if(motorDir == 0){
            motor5 = 0;
            motor6 = 0;
            motor9 = 0;
            motor10 = 0;

        // Error
        }else{
            blinkLed(3); 
      
        }

    }else{
        motorSpeed = motorSpeed * -1;
        
        // Avanzar
        if(motorDir == 1){
            motor5 = 0;
            motor10 = 0;
            motor6 = motorSpeed;
            motor9 = motorSpeed;
               
        // Derecha
        }else if(motorDir == 2 || motorDir == 6){
            motor5 = 0;
            motor6 = motorSpeed;
    
        // Izquierda
        }else if(motorDir == 3 || motorDir == 5){
            motor10 = 0;
            motor9 = motorSpeed;
    
        // Atras
        }else if(motorDir == 4){
            motor6 = 0;
            motor9 = 0;
            motor5 = motorSpeed;
            motor10 = motorSpeed;

        // Para
        }else if(motorDir == 0){
            motor5 = 0;
            motor6 = 0;
            motor9 = 0;
            motor10 = 0;

        // Error
        }else{
            blinkLed(3); 
        
        }
      
    }

    if (motorSteeps >= 0){
        pinMode(2, INPUT);
        pinMode(3, INPUT);
        attachInterrupt(digitalPinToInterrupt(2), runMotorStopWhell1, CHANGE); // Interrupt 0 en nano es el pin 2  CHANGE
        attachInterrupt(digitalPinToInterrupt(3), runMotorStopWhell2, CHANGE); // Interrupt 1 en nano es el pin 3
    }
    
    motorNow5 = motor5;
    motorNow6 = motor6;
    motorNow9 = motor9;
    motorNow10 = motor10;
    analogWrite(5, motorNow5);
    analogWrite(6, motorNow6);
    analogWrite(9, motorNow9);
    analogWrite(10, motorNow10);

}


// Derecha
void runMotorStopWhell1(){
    motorRoundsWhell1++;      

    if (motorRoundsWhell1 > motorRoundsWhellTarget){
        digitalWrite(9, LOW);
        digitalWrite(10, LOW);     
        
        detachInterrupt(digitalPinToInterrupt(2));       
  
    } else {
        if ((motorRoundsWhell1 - motorRoundsWhell2) > 1){
            motorNow9 -= 2;
            motorNow10 -= 2;
        }else{
            if(motorNow9 < motor9){
                motorNow9 += 2;
                motorNow10 += 2;     
            }   
        }
        if(motor9 > 0){
          analogWrite(9, motorNow9);
        }
        if(motor10 > 0){
          analogWrite(10, motorNow10);
        }                          
    }
}


// Izquierda
void runMotorStopWhell2(){
    motorRoundsWhell2++;      

   if (motorRoundsWhell2 > motorRoundsWhellTarget){
        digitalWrite(5, LOW);
        digitalWrite(6, LOW);     
        
        detachInterrupt(digitalPinToInterrupt(3));      
           
    } else {
        if ((motorRoundsWhell2 - motorRoundsWhell1) > 1){
            motorNow5 -= 2;
            motorNow6 -= 2;
            //motorMedia9_10
        }else{
            if(motorNow5 < motor5){
                motorNow5 += 2;
                motorNow6 += 2;
            }        
            //motorNow5  = motor5;
            //motorNow6 = motor6;        
        }
        if(motor5 > 0){
          analogWrite(5, motorNow5);
        }
        if(motor6 > 0){
          analogWrite(6, motorNow6);
        }                          
    }
}



/********************************************************************
 * BlickLed
 */

void blinkLed(int num){
    for(int i=0; i<num; i++){
        digitalWrite(13, HIGH);
        buzzer.tone(8, 250, 200);
        digitalWrite(13, LOW);
        delay(200);              
    }
    delay(500);              
}

